//***********************************************
//*												*
//*           	  Exercise 5-1					*
//*												*
//***********************************************

#include <p18f452.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include "P18F_LCD.h"

void InitializeAD(void);
void InitializePWM2(void);
void InitializeTMR2(void);
void InitializePORT(void);
void InitializeLCD(void);
void LCD_ItoA(unsigned int);
unsigned char Set_BCD_ASCII(unsigned char);

/* define the LCD_MSG1[] in program memory at 0x1000 */
//***************************************************** 
#pragma romdata My_romdata=0x1000
const rom char LCD_MSG1[]="  C18 Workshop ";
#pragma romdata

/* define the LCD_MSG2[] in data memory at 0x100 */
//************************************************* 
#pragma idata My_Banked_RAM=0x100
char LCD_MSG2[]="A/D Result = ";
#pragma idata

/* define following variable in data memory at Access Bank */
//*********************************************************** 
#pragma udata access My_Access_RAM

near unsigned int AD_Temp;
near unsigned char DS_Zero_FLG;

union
{
	unsigned int AD_10bit;    // contant is 10-bit A/D Result
    unsigned char AD[2];	  // AD[0] is A/D Low byte, AD[1] is A/D High Byte
} near AD_Result;

#pragma udata


void main(void)
{
	InitializePORT( );
	InitializePWM2( );
	InitializeAD( );		
	InitializeTMR2( );
	OpenLCD( );

 	LCD_Set_Cursor(0,0);						// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG1);

	LCD_Set_Cursor(1,0);						// Put LCD Cursor on Line 2
 	putsLCD(LCD_MSG2);


		while(1)
	{
		ADCON0bits.GO=1;							// Start to convert the A/D
		while(ADCON0bits.GO);						// Waiting A/D until done

		AD_Temp=ReadADC();							// Get 10 bits A/D result
		AD_Result.AD_10bit=AD_Temp;

		SetDCPWM2(AD_Result.AD_10bit);				// Put 10 bits A/D result to PWM1
//		AD_Result.AD_10bit>>=2; 					// Right shift 2 bits
//		PORTD=AD_Result.AD[0];						// put the MSB to PORTD
	
		LCD_Set_Cursor(1,12);						// Set LCD cursor to Line 2 with location 12
		LCD_ItoA(AD_Temp);							// Show 10-bit A/D Decimal Valum on LCD
	}
}

void LCD_ItoA(unsigned int AD_Data)
{
	
	    DS_Zero_FLG=1;
		putcLCD(Set_BCD_ASCII(AD_Data/1000));		// ܤd
		AD_Temp=AD_Temp%1000;						// XʦH᪺

		putcLCD(Set_BCD_ASCII(AD_Temp/100));		// ܦʦ
		AD_Temp=AD_Temp%100;
	
		putcLCD(Set_BCD_ASCII(AD_Temp/10));			// ܤQ
		AD_Temp=AD_Temp%10;

		putcLCD(AD_Temp+='0');						// ܭӦ
}

unsigned char Set_BCD_ASCII(unsigned char BCD_Data)
{
		if (BCD_Data==0) 
		{
			if (DS_Zero_FLG) return ' ';   		// ~s
			else	return '0';					// ܤ@몺s
		}
		else 
		{
			DS_Zero_FLG=0;						// ~s	
			return (BCD_Data +='0');			// öǦ^ ASCII Code
		}		
}

//*********************************************************
//* create following function in program memory at 0x2000 *
//********************************************************* 
#pragma code My_code_on=0x2000

//************************************************
//* Function: InitializeAD                       *
//*        - Fosc/32 clock source                 *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(ADC_FOSC_32 & ADC_RIGHT_JUST & ADC_1ANA_0REF,
			ADC_CH0 & ADC_INT_OFF);
}
//************************************************
//* Function: InitializePWM2                     *
//*   - Initialize the CCP2 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC2 an output                  *
//*        - Configure CCP2 for PWM              *
//************************************************
void InitializePWM2(void)
{
	TRISCbits.TRISC1=0;	//* setup RC1 for PWM output
	OpenPWM2(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	PORTA=0x00;
	TRISA=0b11011011;	// RA2 as LCD-E control, RA5 as CS Control  for SPI	
	PORTD=0x00;
	TRISD=0x00;		// Set PORTD as Output port
}


#pragma code	



