//***************************************************
//*											     	*
//*           	  Exercise 5-2					    *
//* MPLAB C18 WOrkshop Exercise for High-priority   *
//* Setting with C code.							*
//*												    *
//***************************************************

#include <p18f452.h>
#include <timers.h>
#include <delays.h>

void isr_high(void);

volatile unsigned char Long_Count ;
volatile unsigned char Direct_LED ;
volatile unsigned char Dir_Count ;

void main(void)
{
		TRISD=0x00;					// Set output port for LED driver
		PORTD=0b00000001;			// Set b0 of LED is On

		RCONbits.IPEN=1;			// Enable Interrupt Priority bit
		IPR1bits.TMR2IP=1;			// Set Timer2 for High Priority
		INTCONbits.GIEH=1;			// Enable High Priority Interrupt

	/***********************************/
	/*        Interrupt Time	   */ 		
	/*(1/(16Mhz/4)) (16*10*(199+1)) = 8mS */
	/***********************************/
		OpenTimer2 (	TIMER_INT_ON		// Turn On the Timer2 with Interrupt
				& T2_PS_1_16
				& T2_POST_1_10);

		PR2 = 199;

		Long_Count=0;
		Direct_LED=0;
		Dir_Count=0;

		while(1);				// Loop Here!
}

//************************************************
//*       #pragma Interrupt Declarations         *
//*												 *
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 					//begin in-line assembly
     goto isr_high				//go to isr_high function
     _endasm 					//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//*   High priority interrupt will               *
//*   - Received a serial data from RS-232       *
//*     Save the received data to buffer Rec_Data*                  
//************************************************
#pragma interrupt isr_high 

void isr_high(void)
{
	PIR1bits.TMR2IF=0;			// Clear Timer2 interrupt Flag

	if (Long_Count <= 12) Long_Count++;  	// 8mS * 12 = 96mS
	else
	{	
		Long_Count=0;			// Time is 96mS, do the function

		if (Direct_LED==0x00)		// Right or Left shift
		{
		 	PORTD<<=1;		// LED left shift
			Dir_Count++;
			if (Dir_Count==7) 	// End of LED position?
			{
				Dir_Count=0;	// Yes, set flag of right shift
				Direct_LED=0x1;
			}
		}
		else
		{
			PORTD>>=1;		// LED right shift
			Dir_Count++;
			if (Dir_Count==7) 
			{
				Dir_Count=0;
				Direct_LED=0x00;
			}
		}
	}		
}								
#pragma code







