//***********************************************
//*												*
//*           	  Exercise 7-1					*
//*												*
//***********************************************
/* Include Header files  */

#include <p18f452.h>
#include <adc.h>
#include <pwm.h>
#include <timers.h>
#include <delays.h>
#include <usart.h>
#include <stdlib.h>
#include <ctype.h>
#include <i2c.h>
#include "p18LCD.h"

/* Declare the Function Prototype */
//void InitializeAD(void);
//void InitializePWM1(void);
//void InitializeTMR2(void);
void InitializePORT(void);
void InitializeUSART(void);
void InitializeINTs(void);
void Initialize_I2C_Master(void);
void isr_high_direct(void);
void isr_high(void);
void Display_Terminal(const rom far char *data);
void Send_CR_LF(void);
 

#define NULL 0x00
#define Min_Langth 4

/* define the LCD_MSG1[] in program memory at 0x1000 */
//***************************************************** 
#pragma romdata My_romdata=0x1000
const rom far unsigned char LCD_MSG1[]="Waiting Data Now!";
const rom far unsigned char LCD_MSG2[]="-Display EEPROM- ";

const rom far char Disp_Msg1[]="Please Enter your Number :";
const rom far char Disp_Msg2[]="EEPROM WRITE is completed!"; 
const rom far char Disp_Msg3[]="\aError! Please enter again:";
const rom far char Disp_Msg4[]="\aYou must get 4 Digials at least! ";
const rom far char Disp_Msg5[]="Completed to save your Number!";
#pragma romdata

/* define following variable in data memory at Access Bank */
//*********************************************************** 
#pragma udata access My_RAM_2

near unsigned char f;
near unsigned char j;

union					
{
int l;
char i[2];
} near EE_Read_Data;

#pragma udata

unsigned char Rec_Data;
unsigned char EE_Addr;


//***********************************************************
/*														   */
/* 						Main 	Program					   */
/*														   */
//***********************************************************


void main(void)
{

	InitializePORT( );
//	InitializePWM1( );
//	InitializeAD( );		
//	InitializeTMR2( );
	OpenLCD( );
	Initialize_I2C_Master( );
	InitializeINTs( );							
	InitializeUSART( );

	Display_Terminal(Disp_Msg1);					// Send Disp_Msg1 to Hyper-Terminal

 	LCD_Set_Cursor(0,0);						// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG1);

	Rec_Data=0x00;									// Clear Receive data, it is from RS-232
	EE_Addr=0;										// Set EEPROM start address = 0

	do
	{
		while(Rec_Data==0x00);									// New RS-232 data received ? No, Loop here.
		

			if (isdigit(Rec_Data ))								// Received data is a decimal digit
				{
					f=EEByteWrite (0xA0,EE_Addr,Rec_Data);		// Write Receive Data into EEPROM
					f=EEAckPolling(0xA0);						// Polling the Acknowledge
					EE_Addr++;									// Point to next address of EEPROM 
					Rec_Data=0x00;								
					Display_Terminal(Disp_Msg2);				// Send Disp_Msg2 to Hyper-Terminal
				}
				else if (Rec_Data!=0x0D) 						// Press "Enter" key or alphabetic character
					{
						Display_Terminal(Disp_Msg3);			// alphabetic, display error message to Hyper-Terminal
						Rec_Data=0x00;
					}
					else if (EE_Addr<Min_Langth) 				// "Enter" key, need to check how many digit stored
						{
							Display_Terminal(Disp_Msg4);		// send error message to Hyper_Terminal
							Rec_Data=0x00;
						}
		
		
	} while ((EE_Addr<10) && ((Rec_Data!=0x0d)||(EE_Addr<Min_Langth)));  // Fully check

	{															// Write NULL into EEPROM for a string format
		Rec_Data=NULL;
		f=EEByteWrite (0xA0,EE_Addr,Rec_Data);
		f=EEAckPolling(0xA0);
		Display_Terminal(Disp_Msg5);
	}

	LCD_Set_Cursor(0,0);										// Put LCD Cursor on Line 1
	putrsLCD(LCD_MSG2);							

	LCD_Set_Cursor(1,0);											// Put LCD Cursor on Line 2
	j=0;

	while(1)													// loop here until read the NULL from EEPROM
	{
		EE_Read_Data.l = EERandomRead(0xA0,j);					// Read data from EEPROM

		if (EE_Read_Data.i[0]!=NULL)							// check NULL data?
			{
				WriteDataLCD(EE_Read_Data.i[0]);				// No, display the data to LCD
				j++;											// Point to next address of EEPROM
			}
		else	break;											//Detected NUll, Break out the loop
	}



	while(1);													// End of process, loop
}
//***********************************************
//*   Send a ASCII string to Terminal           *
//***********************************************
void Display_Terminal(const rom far char *data)	
{	
		Send_CR_LF();
		putrsUSART(data);
		Delay10KTCYx(10);
}
void Send_CR_LF(void)
{
		while (BusyUSART());
		WriteUSART('\n');
		while (BusyUSART());
		WriteUSART('\r');
}
//************************************************
//*       #pragma Interrupt Declarations         *
//************************************************
//************************************************
//* Function: isr_high_direct                    *
//*   - Direct execution to the actual           *
//*     high-priority interrupt code.            *
//************************************************
#pragma code isrhighcode = 0x0008

void isr_high_direct(void)	
{  								
     _asm 			//begin in-line assembly
     goto isr_high	//go to isr_high function
     _endasm 		//end in-line assembly
}
#pragma code

//************************************************
//* Function: isr_high(void)                     *     
//************************************************ 
void isr_high(void)
{
//	PIR1bits.RCIF=0;					// Clear RxD Flag
//	Rec_Data=ReadUSART();				// Get RS-232 data
//	PORTD=Rec_Data;						// Display to LEDs


	_asm
 			bcf		PIR1,5,0 				// clear USART interrupt flag (RCIF)
			movff	RCREG,Rec_Data			// move USART data to Rec_Data
 			movff	RCREG,PORTD				// Display USART data on LEDs
 			call	Send_USART,0
 			retfie 	1						// Return with SHADOW swap
 
Send_USART:
			movlw	'='						// Load "=" to Wreg.
			movwf	TXREG,0					// Send to RS-232
			nop
Tx_Loop:	btfss	PIR1,4,0				// Check TXIF rising to Hi
			goto 	Tx_Loop					// Is Low, loop test
			bcf		PIR1,4,0				// Clear TXIF
			movff	RCREG,TXREG				// Send back the response to RS-232
			return 	0						// Retrun with normal

	_endasm	
				
}								

//************************************************
//* Function: Initialize Interrupts              *
//*   - Reset All Interrupr                      *
//*   - Enable HIgh Priority Interrupt           *
//*   - Disable Low Priority Interrupt           *
//************************************************
void InitializeINTs(void) 
{
 	PIR1=0; 						//Clear all Interrupt setting & Flag
	PIR2=0;
	INTCON=0;
	INTCON2=0;
	INTCON3=0;
	PIE1=0;
	PIE2=0;

	RCONbits.IPEN=1;     			//Enable priority interrupts

	INTCONbits.GIEL=0;				// Disable Low Priority Interrupt
	INTCONbits.GIEH=1;				// Enalbe high priority Interrupt
}
//************************************************
//* Initial I2C Master Mode with 7 bits Address  *
//* Clock Speed : 100KHz @4MHz					 *
//************************************************
void Initialize_I2C_Master(void)
{
	OpenI2C(MASTER,SLEW_ON);
	SSPADD= 9;
}
//************************************************
//* Function: Initialize USART                   *
//*   - Initialize the RS-232 communication      *
//*   - Disable Interrupr for TxD                *
//*   - Enable Interrupt fro Rxd                 *
//*   - Format: 9600, N, 8, 1                    *
//************************************************
void InitializeUSART(void)
{
	OpenUSART( USART_TX_INT_OFF 		// Set TXSTA Reg. =0b00100100 
			 & USART_RX_INT_ON			// Set RCSTA Reg. =0b10010000 
		 	 & USART_ASYNCH_MODE
			 & USART_EIGHT_BIT
			 & USART_CONT_RX
			 & USART_BRGH_HIGH
			 ,25);						// Set SPBRG=25, Baud Rate = 9600

	PIE1bits.TXIE=0;
	PIR1bits.RCIF=0;
	IPR1bits.RCIP=1;       				// Set Receive of USART are High priority
	PIE1bits.RCIE=1;					// Enable RxD Interrupt
}

//************************************************
//* Function: InitializeAD                       *
//*        - Fosc/8 clock source                 *
//*        - Right justified                     *
//*        - 1 analog channels, 0 references     *
//*        - Channel 0                           *
//*        - Disable A/D interrupt               *
//************************************************
void InitializeAD(void)
{
	OpenADC(ADC_FOSC_8 & ADC_RIGHT_JUST & ADC_1ANA_0REF,
			ADC_CH0 & ADC_INT_OFF);
}
//************************************************
//* Function: InitializePWM1                     *
//*   - Initialize the CCP1 Module as PWM output *
//*     for controlling motor speed              *
//*     (see p160 MPLAB-CXX Reference Guide)     *
//*        - Set the period to FFh               *
//*        - Make RC2 an output                  *
//*        - Configure CCP1 for PWM              *
//************************************************
void InitializePWM1(void)
{
	TRISCbits.TRISC2=0;	//* setup RC2 for PWM output
	OpenPWM1(0xff); 	//* set period to 0xFF
}


//************************************************
//* Function: InitializeTimer2                   *
//*   - Initialize Timer2 for use with CCP1 PWM  *
//*        - 1:16 Postscale                      *
//*        - 1:4 Prescale                        *
//*        - Enable the timer                    *
//************************************************
void InitializeTMR2(void)
{
	OpenTimer2(T2_PS_1_4 & T2_POST_1_16);
}
//************************************************
//* Function: InitializePORT                     *
//*   - Make PORTD all outputs                   *
//*   - Turn 0ff all the LED's                   *
//************************************************
void InitializePORT(void)
{
	TRISD=0;
	PORTD=0x00;
}
